@@ -39,46 +39,46 @@ pub enum CertificationError {
     #[error("{0}: issuer revoked the certification")]
     IssuerRevoked(Certification),
 
-    #[error("{0}: certification created after reference time ({})",
-            format_time(.1))]
+    #[error("{0}: certification created after reference time ({time})",
+            time=format_time(.1))]
     BornLater(Certification, SystemTime),
 
     /// The certification is expired (1) as of the reference time (2).
-    #[error("{0}: certification expired ({}) as of reference time ({})",
-            format_time(.1), format_time(.2))]
+    #[error("{0}: certification expired ({time1}) as of reference time ({time2})",
+            time1=format_time(.1), time2=format_time(.2))]
     CertificationExpired(Certification, SystemTime, SystemTime),
 
     #[error("{0}: target is not live \
-             as of the certification time ({})",
-            format_time(.1))]
+             as of the certification time ({time})",
+            time=format_time(.1))]
     TargetNotLive(Certification, SystemTime, #[source] anyhow::Error),
 
     #[error("{0}: target certificate is not valid \
-             as of the certification time ({})",
-            format_time(.1))]
+             as of the certification time ({time})",
+            time=format_time(.1))]
     TargetNotValid(Certification, SystemTime, #[source] anyhow::Error),
 
-    #[error("{0}: issuer certificate is hard revoked: {1} ({})",
-            String::from_utf8_lossy(.2))]
+    #[error("{0}: issuer certificate is hard revoked: {1} ({msg})",
+            msg=String::from_utf8_lossy(.2))]
     IssuerHardRevoked(Certification,
                       openpgp::types::ReasonForRevocation, Vec<u8>),
 
     #[error("{0}: issuer certificate is soft revoked \
-             as of the certification time ({}): {2} ({})",
-            format_time(.1),
-            String::from_utf8_lossy(.3))]
+             as of the certification time ({time}): {2} ({msg})",
+            time=format_time(.1),
+            msg=String::from_utf8_lossy(.3))]
     IssuerSoftRevoked(Certification, SystemTime,
                       openpgp::types::ReasonForRevocation, Vec<u8>),
 
-    #[error("{0}: target certificate is hard revoked: {1} ({})",
-            String::from_utf8_lossy(.2))]
+    #[error("{0}: target certificate is hard revoked: {1} ({msg})",
+            msg=String::from_utf8_lossy(.2))]
     TargetHardRevoked(Certification,
                       openpgp::types::ReasonForRevocation, Vec<u8>),
 
     #[error("{0}: target certificate is soft revoked \
-             as of the certification time ({}): {2} ({})",
-            format_time(.1),
-            String::from_utf8_lossy(.3))]
+             as of the certification time ({time}): {2} ({msg})",
+            time=format_time(.1),
+            msg=String::from_utf8_lossy(.3))]
     TargetSoftRevoked(Certification, SystemTime,
                       openpgp::types::ReasonForRevocation, Vec<u8>),
 }
