@@ -23,8 +23,8 @@ use crate::Certification;
 use crate::CertificationSet;
 use crate::Depth;
 use crate::format_time;
+use crate::Network;
 use crate::Path;
-use crate::Query;
 use crate::store::Backend;
 use crate::store::Store;
 
@@ -32,7 +32,7 @@ use super::TRACE;
 
 const NP: NullPolicy = NullPolicy::new();
 
-/// [`Query::lint_path`] specific error codes.
+/// [`Network::lint_path`] specific error codes.
 #[non_exhaustive]
 #[derive(thiserror::Error, Debug)]
 pub enum PathError {
@@ -41,9 +41,9 @@ pub enum PathError {
     EmptyPath,
 
     /// There is a path, but the trust amount is insufficient.
-    #[error("The path {}, {1:?} exists, but its trust amount \
+    #[error("The path {path}, {1:?} exists, but its trust amount \
              is too low ({2}, required: {3}).",
-            .0.iter()
+            path=.0.iter()
                 .map(|kh| KeyID::from(kh).to_hex())
                 .collect::<Vec<String>>()
                 .join(" -> "))]
@@ -60,56 +60,63 @@ pub enum PathError {
     MissingTarget(KeyHandle),
 
     /// The target certificate is expired.
-    #[error("The target ({0}) is expired ({}) as of the reference time ({})",
-            format_time(.1), format_time(.2))]
+    #[error("The target ({0}) is expired ({time1}) \
+             as of the reference time ({time2})",
+            time1=format_time(.1), time2=format_time(.2))]
     TargetExpired(KeyHandle, SystemTime, SystemTime),
 
     /// The target certificate is revoked.
     ///
     /// 0. Target cert, 1. revocation code, 2. revocation reason,
     /// 3. revocation time, 4. reference time.
-    #[error("The target ({0}) is revoked ({}, {1}, {}) \
-             as of the reference time ({})",
-            format_time(.3), String::from_utf8_lossy(.2), format_time(.4))]
+    #[error("The target ({0}) is revoked ({time1}, {1}, {msg}) \
+             as of the reference time ({time2})",
+            time1=format_time(.3),
+            msg=String::from_utf8_lossy(.2),
+            time2=format_time(.4))]
     TargetRevoked(KeyHandle, ReasonForRevocation, Vec<u8>, SystemTime, SystemTime),
 
     /// The target User ID is revoked.
     ///
     /// 0. Target cert, 1. target user id, 2. revocation code,
     /// 3. revocation reason, 4. revocation time, 5. reference time.
-    #[error("The target's ({0}) User ID ({}) is revoked ({}, {2}, {}) \
-             as of the reference time ({})",
-            String::from_utf8_lossy(.1.value()), format_time(.4),
-            String::from_utf8_lossy(.3), format_time(.5))]
+    #[error("The target's ({0}) User ID ({userid:?}) is revoked \
+             ({time1}, {2}, {msg}) as of the reference time ({time2})",
+            userid=String::from_utf8_lossy(.1.value()),
+            time1=format_time(.4),
+            msg=String::from_utf8_lossy(.3),
+            time2=format_time(.5))]
     TargetUserIDRevoked(KeyHandle, UserID, ReasonForRevocation, Vec<u8>,
                         SystemTime, SystemTime),
 
     /// None of this certificate's regular expressions match the
     /// target User ID.
-    #[error("None of the certification's ({0}) regular expressions ({:?}) match \
-             the target User ID ({:?})",
-            .0.regular_expressions_bytes()
+    #[error("None of the certification's ({0}) regular expressions ({regex:?}) \
+             match the target User ID ({userid:?})",
+            regex=.0.regular_expressions_bytes()
                 .iter()
                 .map(|re| {
                     String::from_utf8_lossy(re).into_owned()
                 })
                 .collect::<Vec<String>>()
                 .join(", "),
-            String::from_utf8_lossy(.1.value()))]
+            userid=String::from_utf8_lossy(.1.value()))]
     RegexMismatch(Certification, UserID),
 
     /// The certification's target does not have a sufficiently high
     /// trust depth to authenticate the rest of the path.
-    #[error("The path requires that {} be a level-{1} \
+    #[error("The path requires that {keyid} be a level-{1} \
              trusted introducer, the certification {0} only makes it \
-             a level-{} trusted introducer",
-            .0.target().keyid().to_hex(), .0.depth())]
+             a level-{depth} trusted introducer",
+            keyid=.0.target().keyid().to_hex(),
+            depth=.0.depth())]
     InsufficientTrustDepth(Certification, Depth),
 
     /// The certification does not have a sufficient trust amount.
     #[error("The path requires a trust amount of {1}, but \
-             this certification ({0}) only has a trust amount of {}",
-            .0.amount())]
+             this certification ({0}) only has a trust amount of \
+             {amount}",
+            amount=.0.amount())]
     InsufficientTrustAmount(Certification, usize),
 
     /// The certificate did not issue a certification for the
@@ -126,18 +133,18 @@ pub enum PathError {
 
     /// None of the active certifications were adequate for
     /// authenticating the target.
-    #[error("No active certifications by {} for <{}, {:?}> \
-             had a trust amount of at least {3}",
-            .0.keyid(), .1.keyid(),
-            String::from_utf8_lossy(.2.value()))]
+    #[error("No active certifications by {keyid1} for \
+             <{keyid2}, {userid:?}> had a trust amount of at least {3}",
+            keyid1=.0.keyid(), keyid2=.1.keyid(),
+            userid=String::from_utf8_lossy(.2.value()))]
     NoAdequateCertification(CertSynopsis, CertSynopsis, UserID, usize),
 
     /// None of the active delegations were adequate for the path
     /// suffix.
-    #[error("No active certifications by {} for {} \
+    #[error("No active certifications by {keyid1} for {keyid2} \
              that make it at least a level-{2} trusted introducer \
              with a trust amount of at least {3}",
-            .0.keyid(), .1.keyid())]
+            keyid1=.0.keyid(), keyid2=.1.keyid())]
     NoAdequateDelegation(CertSynopsis, CertSynopsis, Depth, usize),
 
     /// A certification would be adequate, but it is not active.
@@ -213,7 +220,7 @@ impl<'a> CertVariants<'a> {
 ///
 /// Sometimes we only know its Key ID.
 ///
-/// This is indirectly returned by [`Query::lint_path`].
+/// This is indirectly returned by [`Network::lint_path`].
 ///
 /// The lifetimes `'a` and `'b` have the same meaning as for
 /// [`Store`]: `'a` is the lifetime of the object on the backend, `'b`
@@ -301,7 +308,7 @@ enum CertificationVariants {
 /// Sometimes we only know the Key ID of the issuer and the Key ID of
 /// the target.
 ///
-/// This is indirectly returned by [`Query::lint_path`].
+/// This is indirectly returned by [`Network::lint_path`].
 #[derive(Debug)]
 pub struct CertificationLints {
     certification: CertificationVariants,
@@ -464,7 +471,7 @@ impl CertificationLints {
 
 /// A linted path.
 ///
-/// This is returned by [`Query::lint_path`].
+/// This is returned by [`Network::lint_path`].
 ///
 /// The lifetimes `'a` and `'b` have the same meaning as for
 /// [`Store`]: `'a` is the lifetime of the object on the backend, `'b`
@@ -548,11 +555,49 @@ impl<'a> PathLints<'a> {
                         c.amount()
                     } else if c.depth() < required_depth.into() {
                         0
-                    } else if ! c.regular_expressions()
-                        .map(|re_set| re_set.matches_userid(&userid))
-                        // Invalid => assume everything matches.
-                        .unwrap_or(true)
+                    } else if required_depth > 0
+                        && (! c.regular_expressions()
+                            .map(|re_set| {
+                                let matches = re_set.matches_userid(&userid);
+                                t!("Certification's regular expression \
+                                    {} target user ID {}",
+                                   if matches {
+                                       "matches"
+                                   } else {
+                                       "does not match"
+                                   },
+                                   String::from_utf8_lossy(userid.value()));
+                                matches
+                            })
+                            // Invalid => assume everything matches.
+                            .unwrap_or(true))
                     {
+                        // We check that the current certificate's
+                        // regular expressions match the target user
+                        // ID UNLESS (`required_depth == 0`) this is
+                        // the certification that introduces the
+                        // target user ID.
+                        //
+                        // Consider: Alice delegates to 'Bob
+                        // <bob@other.org>', but only for "some.org".
+                        // Bob's email address (`bob@other.org`) is
+                        // not in some.org, but that doesn't matter
+                        // when considering Alice's introduction of
+                        // Bob; the regular expressions only scopes
+                        // what user IDs Bob can introduce!
+                        //
+                        // ```
+                        //     Alice <alice@example.org>
+                        //     |
+                        //     | Authorization (depth: 1, amount: 120),
+                        //     | Regular expression: some.org
+                        //     v
+                        //     Bob <bob@other.org>
+                        //    /                   \
+                        //   / Certification       \ Certification
+                        // v                        v
+                        // Carol <carol@some.org>   Dave <dave@other.org>
+                        // ```
                         0
                     } else {
                         c.amount()
@@ -567,7 +612,7 @@ impl<'a> PathLints<'a> {
     ///
     /// This fails if the path is invalid.  Note: the path is still
     /// considered valid even if it doesn't have the required trust
-    /// amount as passed to [`Query::lint_path`].
+    /// amount as passed to [`Network::lint_path`].
     ///
     /// There may be multiple reasons why a path is invalid.  This
     /// function tries to return the first (when checking from the
@@ -646,7 +691,7 @@ impl<'a> From<&Path> for PathLints<'a> {
 /// [`Store`]: `'a` is the lifetime of the object on the backend, `'b`
 /// is the lifetime of the reference to the object, and the object
 /// (`'a`) must outlive the reference (`'b`).
-impl<'a: 'b, 'b, S> Query<'b, S>
+impl<'a, S> Network<S>
     where S: Store + Backend<'a>
 {
     /// Authenticates a path in the network.
@@ -658,15 +703,15 @@ impl<'a: 'b, 'b, S> Query<'b, S>
     /// This function will return `Ok` if a path with the required
     /// trust amount can be found.
     ///
-    /// Unlike [`Query::lint_path`], this function returns as soon
+    /// Unlike [`Network::lint_path`], this function returns as soon
     /// as an error is encountered.
     ///
     /// This function requires that the [`Network`] object implement
     /// [`Backend`] in addition to [`Store`].  This is technically
-    /// needed by [`Query::lint_path`] to provide better diagnostics,
-    /// but it is not strictly required by [`Query::path`], which
+    /// needed by [`Network::lint_path`] to provide better diagnostics,
+    /// but it is not strictly required by [`Network::path`], which
     /// only needs active certifications.  This requirement exists,
-    /// because [`Query::path`] and [`Query::lint_path`] share a
+    /// because [`Network::path`] and [`Network::lint_path`] share a
     /// fair amount of code.  This bound may be lifted in the future.
     ///
     /// [`Network`]: crate::Network
@@ -708,7 +753,7 @@ impl<'a: 'b, 'b, S> Query<'b, S>
     /// This function almost always returns `Ok`; it only returns an
     /// error in an extraordinary circumstance.
     ///
-    /// Unlike [`Query::path`], this function does extra work to
+    /// Unlike [`Network::path`], this function does extra work to
     /// identify reasons why a path is invalid.  For instance, if
     /// there is no valid certification for a path segment, but there
     /// is an expired certification that is expired, this function
@@ -744,7 +789,7 @@ impl<'a: 'b, 'b, S> Query<'b, S>
         let khs: &[KeyHandle] = khs;
 
         // XXX: let policy = self.policy();
-        let reference_time = self.network().reference_time();
+        let reference_time = self.reference_time();
 
         // Look up the certificates.
         let mut cert_lints: Vec<CertLints> = Vec::with_capacity(khs.len());
@@ -757,7 +802,7 @@ impl<'a: 'b, 'b, S> Query<'b, S>
             certs.push(None);
             let cert = certs.last_mut().expect("have one");
 
-            match self.network().lookup_by_cert(kh) {
+            match self.lookup_by_cert(kh) {
                 Ok(certs) => {
                     if certs.len() > 1 {
                         // XXX: keyid collision :/.  Silently ignore for
@@ -1143,11 +1188,36 @@ impl<'a: 'b, 'b, S> Query<'b, S>
             let adequate = |certification: &Certification,
                             cl: Option<&mut CertificationLints>|
             {
-                if ! self.certification_network()
-                    && ! certification.regular_expressions()
-                             .map(|re_set| re_set.matches_userid(&userid))
-                             // Invalid => everything matches.
-                             .unwrap_or(true)
+                // We check that the current certificate's regular
+                // expressions match the target user ID UNLESS
+                // (`required_depth == 0`) this is the certification
+                // that introduces the target user ID.
+                //
+                // Consider: Alice delegates to 'Bob <bob@other.org>',
+                // but only for "some.org".  Bob's email address
+                // (`bob@other.org`) is not in some.org, but that
+                // doesn't matter when considering Alice's
+                // introduction of Bob; the regular expressions only
+                // scopes what user IDs Bob can introduce!
+                //
+                // ```
+                //     Alice <alice@example.org>
+                //     |
+                //     | Authorization (depth: 1, amount: 120),
+                //     | Regular expression: some.org
+                //     v
+                //     Bob <bob@other.org>
+                //    /                   \
+                //   / Certification       \ Certification
+                // v                        v
+                // Carol <carol@some.org>   Dave <dave@other.org>
+                // ```
+                let result = if ! self.certification_network()
+                    && (required_depth > 0.into()
+                        && (! certification.regular_expressions()
+                            .map(|re_set| re_set.matches_userid(&userid))
+                            // Invalid => everything matches.
+                            .unwrap_or(true)))
                 {
                     if let Some(cl) = cl {
                         let err = PathError::RegexMismatch(
@@ -1175,10 +1245,14 @@ impl<'a: 'b, 'b, S> Query<'b, S>
                     }
                     false
                 } else {
-                    t!("  Found an adequate certification: {:?}",
-                       certification);
                     true
-                }
+                };
+
+                t!("  Certification is {}adequate: {:?}",
+                   if result { "" } else { "not " },
+                   certification);
+
+                result
             };
 
             // The CertificationSet returns the active
@@ -1186,18 +1260,13 @@ impl<'a: 'b, 'b, S> Query<'b, S>
             // form.
             let cs = CertificationSet::from_certifications(
                 valid_certifications.clone(), reference_time);
-            let active_certifications: Vec<&Certification> = cs
-                .iter()
-                .flat_map(|cs| {
-                    cs.certifications()
-                        .flat_map(|(_userid, certifications)| {
-                            certifications
-                        })
-                })
+            let active_certifications: Vec<Certification> = cs
+                .into_iter()
+                .flat_map(|cs| cs.into_certifications())
                 .collect();
             t!("Have {} active certifications", active_certifications.len());
-            for &certification in active_certifications.iter() {
-                if adequate(certification, None) {
+            for certification in active_certifications.iter() {
+                if adequate(&certification, None) {
                     // We found a good certification /
                     // delegation.  We're done, even if we are
                     // linting.
