@@ -152,7 +152,9 @@ impl Path {
     /// Returns an iterator over the path's certificates (the nodes).
     ///
     /// The certificates are returned from the root towards the target.
-    pub fn certificates<'a>(&'a self) -> impl Iterator<Item=&CertSynopsis> + 'a {
+    pub fn certificates<'a>(&'a self)
+        -> impl 'a + Iterator<Item=&CertSynopsis> + DoubleEndedIterator
+    {
         std::iter::once(&self.root)
             .chain(self.edges.iter().map(|certification| {
                 certification.target()
@@ -277,6 +279,65 @@ impl Path {
 
         Ok(())
     }
+
+    /// Returns whether the path describes a self signature.
+    pub fn self_signature(&self) -> bool {
+        self.edges.is_empty()
+    }
+
+    /// Returns whether the specified path is a suffix of the path.
+    ///
+    /// If two paths are the same, then this returns true.
+    ///
+    /// A self signature is not considered a suffix of non-self
+    /// signature.  That is, a path describing a self-signature for
+    /// Alice is different from Bob certifying Alice's certificate.
+    ///
+    /// This checks that the certificates along the path are the same.
+    /// It does *not* check that the certification parameters (the
+    /// trust amount, the regular expressions, etc.) or that the user
+    /// IDs (including the target user ID) are the same.
+    fn has_suffix(&self, path: &Path) -> bool {
+        tracer!(false, "Path::has_suffix");
+        t!("Self: {:?}", self);
+        t!("Other: {:?}", path);
+
+        if self.len() < path.len() {
+            // `self` is shorter than `path`.  `path` can't be a
+            // suffix of `self`.
+            t!("self is shorter ({}) than path ({}); path can't be a suffix.",
+               self.len(), path.len());
+            return false;
+        }
+
+        if self.self_signature() != path.self_signature() {
+            t!("self is{} a self signature, path is{}",
+               if self.self_signature() {
+                   ""
+               } else {
+                   " not"
+               },
+               if path.self_signature() {
+                   ""
+               } else {
+                   " not"
+               });
+            return false;
+        }
+
+        let certs_match = self.certificates().rev()
+            .zip(path.certificates().rev())
+            .all(|(this, other)| {
+                this.fingerprint() == other.fingerprint()
+            });
+        if certs_match {
+            t!("Certificates match");
+            true
+        } else {
+            t!("Certificates don't match");
+            return false;
+        }
+    }
 }
 
 /// A collection of paths.
@@ -358,6 +419,15 @@ impl Paths {
     pub fn push(&mut self, path: Path, amount: usize) {
         self.paths.push((path, amount));
     }
+
+    /// Returns whether the specified path is a suffix of an existing
+    /// path.
+    ///
+    /// This only compares the certificates; it does not compare the
+    /// trust values.
+    pub fn has_suffix(&self, path: &Path) -> bool {
+        self.iter().any(|(other, _amount)| other.has_suffix(path))
+    }
 }
 
 impl Deref for Paths {
@@ -383,3 +453,107 @@ impl fmt::Debug for Paths {
         Ok(())
     }
 }
+
+#[cfg(test)]
+mod test {
+    use super::*;
+
+    use std::time::SystemTime;
+
+    use sequoia_openpgp as openpgp;
+    use openpgp::Fingerprint;
+    use sequoia_openpgp::packet::UserID;
+    use openpgp::types::RevocationStatus;
+
+    #[allow(unused)]
+    #[test]
+    fn has_suffix() {
+        let alice_fpr: Fingerprint =
+            "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
+            .parse().expect("valid fingerprint");
+        let alice_uid = UserID::from("<alice@example.org>");
+
+        let alice = CertSynopsis::new(
+            alice_fpr.clone(), None,
+            RevocationStatus::NotAsFarAsWeKnow.into(),
+            std::iter::once((alice_uid.clone(), SystemTime::now())));
+
+        let bob_fpr: Fingerprint =
+            "BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB"
+            .parse().expect("valid fingerprint");
+        let bob_uid = UserID::from("<bob@example.org>");
+
+        let bob = CertSynopsis::new(
+            bob_fpr.clone(), None,
+            RevocationStatus::NotAsFarAsWeKnow.into(),
+            std::iter::once((bob_uid.clone(), SystemTime::now())));
+
+        let carol_fpr: Fingerprint =
+            "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC"
+            .parse().expect("valid fingerprint");
+        let carol_uid = UserID::from("<carol@example.org>");
+
+        let carol = CertSynopsis::new(
+            carol_fpr.clone(), None,
+            RevocationStatus::NotAsFarAsWeKnow.into(),
+            std::iter::once((carol_uid.clone(), SystemTime::now())));
+
+        let t = SystemTime::now();
+
+        // Alice certifies Bob.
+        let alice_certifies_bob
+            = Certification::new(alice.clone(),
+                                 Some(bob_uid.clone()),
+                                 bob.clone(),
+                                 t);
+
+        // Alice certifies Carol.
+        let alice_certifies_carol
+            = Certification::new(alice.clone(),
+                                 Some(carol_uid.clone()),
+                                 carol.clone(),
+                                 t);
+
+        // Bob certifies Carol.
+        let bob_certifies_carol
+            = Certification::new(bob.clone(),
+                                 Some(carol_uid.clone()),
+                                 carol.clone(),
+                                 t);
+
+        let mut alice_bob_carol = Path::new(alice.clone());
+        alice_bob_carol.append(alice_certifies_bob.clone());
+        alice_bob_carol.append(bob_certifies_carol.clone());
+
+        let mut alice_bob = Path::new(alice.clone());
+        alice_bob.append(alice_certifies_bob.clone());
+
+        let mut alice_carol = Path::new(alice.clone());
+        alice_carol.append(alice_certifies_carol.clone());
+
+        let mut bob_carol = Path::new(bob.clone());
+        bob_carol.append(bob_certifies_carol.clone());
+
+        // The following are self signatures and thus aren't suffixes
+        // of longer paths.
+        let mut alice = Path::new(alice.clone());
+        let mut bob = Path::new(bob.clone());
+        let mut carol = Path::new(carol.clone());
+
+        assert!(! alice.has_suffix(&bob));
+        assert!(alice.has_suffix(&alice));
+
+        assert!(! alice_bob.has_suffix(&alice));
+        assert!(alice_bob.has_suffix(&alice_bob));
+        assert!(! alice_bob.has_suffix(&alice_carol));
+        assert!(! alice_bob.has_suffix(&bob));
+
+        assert!(! alice_bob_carol.has_suffix(&alice));
+        assert!(! alice_bob_carol.has_suffix(&bob));
+        assert!(! alice_bob_carol.has_suffix(&carol));
+        assert!(! alice_bob_carol.has_suffix(&alice_bob));
+        assert!(! alice_bob.has_suffix(&alice_carol));
+        assert!(alice_bob_carol.has_suffix(&alice_bob_carol));
+        assert!(alice_bob_carol.has_suffix(&bob_carol));
+    }
+}
