@@ -728,11 +728,23 @@ impl<'a: 'ra, 'ra, S> Store for CertStore<'a, 'ra, S>
             .flat_map(|cert| {
                 cert.userids()
                     .filter_map(|userid| {
+                        // We want to compare the normalized forms.
+                        // But if they already match, then they will
+                        // normalize to the same thing, and we can
+                        // avoid the cost of doing the normalization.
                         if let Ok(Some(e)) = userid.email2() {
                             if e == email {
                                 Some((cert.fingerprint(), userid.clone()))
                             } else {
-                                None
+                                if let Ok(Some(e)) = userid.email_normalized() {
+                                    if e == email {
+                                        Some((cert.fingerprint(), userid.clone()))
+                                    } else {
+                                        None
+                                    }
+                                } else {
+                                    None
+                                }
                             }
                         } else {
                             None
@@ -906,4 +918,31 @@ mod test {
 
         Ok(())
     }
+
+    #[test]
+    fn email_lookup() -> Result<()> {
+        use crate::store::Store as _;
+
+        let certs = CertParser::from_bytes(
+            &crate::testdata::data("puny-code.pgp"))?
+            .collect::<Result<Vec<Cert>>>()?;
+        let store = CertStore::from_cert_refs(
+            certs.iter(), P, None)?;
+
+        let certs = store.lookup_synopses_by_email("hÄNS@bücher.tld");
+        assert_eq!(certs.len(), 1);
+
+        // Not a valid email address.
+        let certs = store.lookup_synopses_by_email("<hÄNS@bücher.tld>");
+        assert_eq!(certs.len(), 0);
+
+        // Make sure we compare on the normalized email address.
+        let certs = store.lookup_synopses_by_email("hÄns@bücher.tld");
+        assert_eq!(certs.len(), 1);
+
+        let certs = store.lookup_synopses_by_email("häns@xn--bcher-kva.tld");
+        assert_eq!(certs.len(), 1);
+
+        Ok(())
+    }
 }
